--[[
	Copyright (C) 2006-2007 Nymbia

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]
local L = AceLibrary("AceLocale-2.2"):new("Quartz")
local BS = AceLibrary("Babble-Spell-2.2")
local sm = AceLibrary("SharedMedia-1.0")
local autoshotname = BS["Auto Shot"]
local resetspells = {
	[BS["Cleave"]] = true,
	[BS["Heroic Strike"]] = true,
	[BS["Maul"]] = true,
	[BS["Raptor Strike"]] = true,
	[BS["Slam"]] = true,
}

local unpack = unpack
local math_abs = math.abs

local Quartz = Quartz
if Quartz:HasModule('Swing') then
	return
end
local QuartzSwing = Quartz:NewModule('Swing')
local QuartzPlayer = Quartz:GetModule('Player')

local GetTime = GetTime

local swingbar, swingbar_width, swingstatusbar, remainingtext, durationtext, db
local swingmode -- nil is none, 0 is meleeing, 1 is autoshooting
local starttime, duration

local BOOKTYPE_SPELL = BOOKTYPE_SPELL

local function OnUpdate()
	local spent = GetTime() - starttime
	remainingtext:SetText(('%.1f'):format(duration - spent))
	local perc = spent / duration
	if perc > 1 then
		return swingbar:Hide()
	else
		swingstatusbar:SetValue(perc)
	end
end
local function OnHide()
	swingbar:SetScript('OnUpdate', nil)
	usingspell = nil
end
local function OnShow()
	swingbar:SetScript('OnUpdate', OnUpdate)
end

function QuartzSwing:OnInitialize()
	db = Quartz:AcquireDBNamespace("Swing")
	Quartz:RegisterDefaults("Swing", "profile", {
		barcolor = {1, 1, 1},
		swingalpha = 1,
		swingheight = 4,
		swingposition = L["Top"],
		swinggap = -4,
		
		durationtext = true,
		remainingtext = true,
	})
end
function QuartzSwing:OnEnable()
	-- fired when autoattack is enabled/disabled.
	self:RegisterEvent("PLAYER_ENTER_COMBAT")
	self:RegisterEvent("PLAYER_LEAVE_COMBAT")
	-- fired when autoshot (or autowand) is enabled/disabled
	self:RegisterEvent("START_AUTOREPEAT_SPELL")
	self:RegisterEvent("STOP_AUTOREPEAT_SPELL")
	
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS")
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES")
	
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	
	self:RegisterEvent("UNIT_ATTACK")
	if not swingbar then
		swingbar = CreateFrame('Frame', 'QuartzSwingBar', UIParent)
		swingbar:SetFrameStrata('HIGH')
		swingbar:SetScript('OnShow', OnShow)
		swingbar:SetScript('OnHide', OnHide)
		
		swingstatusbar = CreateFrame("StatusBar", nil, swingbar)
		
		durationtext = swingstatusbar:CreateFontString(nil, 'OVERLAY')
		remainingtext = swingstatusbar:CreateFontString(nil, 'OVERLAY')
		swingbar:Hide()
	end
	Quartz.ApplySettings()
end
function QuartzSwing:OnDisable()
	swingbar:Hide()
end
function QuartzSwing:PLAYER_ENTER_COMBAT()
	local _,_,offhandlow, offhandhigh = UnitDamage('player')
	if math_abs(offhandlow - offhandhigh) <= 0.1 then
		swingmode = 0 -- shouldn't be dual-wielding
	end
end
function QuartzSwing:PLAYER_LEAVE_COMBAT()
	if not swingmode or swingmode == 0 then
		swingmode = nil
	end
end
function QuartzSwing:START_AUTOREPEAT_SPELL()
	swingmode = 1
end
function QuartzSwing:STOP_AUTOREPEAT_SPELL()
	if not swingmode or swingmode == 1 then
		swingmode = nil
	end
end
function QuartzSwing:CHAT_MSG_COMBAT_SELF_HITS(msg)
	if swingmode == 0 then
		self:MeleeSwing()
	end
end
function QuartzSwing:CHAT_MSG_COMBAT_SELF_MISSES(msg)
	if swingmode == 0 then
		self:MeleeSwing()
	end
end
function QuartzSwing:UNIT_SPELLCAST_SUCCEEDED(unit, spell)
	if swingmode == 0 then
		if resetspells[spell] then
			self:MeleeSwing()
		end
	elseif swingmode == 1 then
		if spell == autoshotname then
			self:Shoot()
		end
	end
end
function QuartzSwing:UNIT_ATTACK(unit)
	if unit == 'player' then
		if not swingmode then
			return
		elseif swingmode == 0 then
			duration = UnitAttackSpeed('player')
		else
			duration = UnitRangedDamage('player')
		end
		durationtext:SetText(('%.1f'):format(duration))
	end
end
function QuartzSwing:MeleeSwing()
	duration = UnitAttackSpeed('player')
	durationtext:SetText(('%.1f'):format(duration))
	starttime = GetTime()
	swingbar:Show()
end
function QuartzSwing:Shoot()
	duration = UnitRangedDamage('player')
	durationtext:SetText(('%.1f'):format(duration))
	starttime = GetTime()
	swingbar:Show()
end
function QuartzSwing:ApplySettings()
	if swingbar and Quartz:IsModuleActive('Swing') then
		local ldb = db.profile
		swingbar:ClearAllPoints()
		swingbar:SetHeight(ldb.swingheight)
		swingbar_width = QuartzCastBar:GetWidth() - 8
		swingbar:SetWidth(swingbar_width)
		swingbar:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
		swingbar:SetBackdropColor(0,0,0)
		swingbar:SetAlpha(ldb.swingalpha)
		swingbar:SetScale(QuartzPlayer.db.profile.scale)
		if ldb.swingposition == L["Bottom"] then
			swingbar:SetPoint("TOP", QuartzCastBar, "BOTTOM", 0, -1 * ldb.swinggap)
		else -- L["Top"]
			swingbar:SetPoint("BOTTOM", QuartzCastBar, "TOP", 0, ldb.swinggap)
		end
		
		swingstatusbar:SetAllPoints(swingbar)
		swingstatusbar:SetStatusBarTexture(sm:Fetch('statusbar', Quartz:GetModule('Player').db.profile.texture))
		swingstatusbar:SetStatusBarColor(unpack(ldb.barcolor))
		swingstatusbar:SetMinMaxValues(0, 1)
		
		if ldb.durationtext then
			durationtext:Show()
			durationtext:ClearAllPoints()
			durationtext:SetPoint('BOTTOMLEFT', swingbar, 'BOTTOMLEFT')
			durationtext:SetJustifyH("LEFT")
		else
			durationtext:Hide()
		end
		durationtext:SetFont(sm:Fetch('font', QuartzPlayer.db.profile.font), 9)
		durationtext:SetShadowColor( 0, 0, 0, 1)
		durationtext:SetShadowOffset( 0.8, -0.8 )
		durationtext:SetTextColor(1,1,1)
		durationtext:SetNonSpaceWrap(false)
		durationtext:SetWidth(swingbar_width)
		
		if ldb.remainingtext then
			remainingtext:Show()
			remainingtext:ClearAllPoints()
			remainingtext:SetPoint('BOTTOMRIGHT', swingbar, 'BOTTOMRIGHT')
			remainingtext:SetJustifyH("RIGHT")
		else
			remainingtext:Hide()
		end
		remainingtext:SetFont(sm:Fetch('font', QuartzPlayer.db.profile.font), 9)
		remainingtext:SetShadowColor( 0, 0, 0, 1)
		remainingtext:SetShadowOffset( 0.8, -0.8 )
		remainingtext:SetTextColor(1,1,1)
		remainingtext:SetNonSpaceWrap(false)
		remainingtext:SetWidth(swingbar_width)
	end
end
do
	local function set(field, value)
		db.profile[field] = value
		Quartz.ApplySettings()
	end
	local function get(field)
		return db.profile[field]
	end
	local function setcolor(field, ...)
		db.profile[field] = {...}
		Quartz.ApplySettings()
	end
	local function getcolor(field)
		return unpack(db.profile[field])
	end
	Quartz.options.args.Swing = {
		type = 'group',
		name = L["Swing"],
		desc = L["Swing"],
		order = 600,
		args = {
			toggle = {
				type = 'toggle',
				name = L["Enable"],
				desc = L["Enable"],
				get = function()
					return Quartz:IsModuleActive('Swing')
				end,
				set = function(v)
					Quartz:ToggleModuleActive('Swing', v)
				end,
				order = 100,
			},
			barcolor = {
				type = 'color',
				name = L["Bar Color"],
				desc = L["Set the color of the swing timer bar"],
				get = getcolor,
				set = setcolor,
				passValue = 'barcolor',
				order = 103,
			},
			swingheight = {
				type = 'range',
				name = L["Height"],
				desc = L["Set the height of the swing timer bar"],
				min = 1,
				max = 20,
				step = 1,
				get = get,
				set = set,
				passValue = 'swingheight',
				order = 104,
			},
			swingalpha = {
				type = 'range',
				name = L["Alpha"],
				desc = L["Set the alpha of the swing timer bar"],
				min = 0.05,
				max = 1,
				step = 0.05,
				isPercent = true,
				get = get,
				set = set,
				passValue = 'swingalpha',
				order = 105,
			},
			swingposition = {
				type = 'text',
				name = L["Bar Position"],
				desc = L["Set the position of the swing timer bar"],
				get = get,
				set = set,
				passValue = 'swingposition',
				validate = {L["Top"], L["Bottom"]},
				order = 106,
			},
			swinggap = {
				type = 'range',
				name = L["Gap"],
				desc = L["Tweak the distance of the swing timer bar from the cast bar"],
				min = -10,
				max = 10,
				step = 1,
				get = get,
				set = set,
				passValue = 'swinggap',
				order = 107,
			},
			durationtext = {
				type = 'toggle',
				name = L["Duration Text"],
				desc = L["Toggle display of text showing your total swing time"],
				get = get,
				set = set,
				passValue = 'durationtext',
				order = 108,
			},
			remainingtext = {
				type = 'toggle',
				name = L["Remaining Text"],
				desc = L["Toggle display of text showing the time remaining until you can swing again"],
				get = get,
				set = set,
				passValue = 'remainingtext',
				order = 109,
			},
		},
	}
end